from os.path import join


def crossminer_features():
    """"""
    return {"ring": "apolar",
            "ring_planar_projected": "apolar",
            "ring_non_planar": "apolar",
            "hydrophobic": "apolar",
            "donor_ch_projected": "donor",
            "donor_projected": "donor",
            "donor": "donor",
            "acceptor_projected": "acceptor",
            "acceptor": "acceptor",
            "negative": "",
            "positive": ""}


def superstar_ins(jobname, probename, settings):
    ss_str = '''JOBNAME {0}
PROBENAME {1}
GRID_SPACING 0.5
CALC_CONTOUR_SURFACE 0
SAVE_SUPER_SCATTERPLOT 0
RUN_TYPE SUPERSTAR
DATA_SOURCE CSD
SHELL_VALUES 0.5 0
SIGCHECK_METHOD POISSON SHELL
PROPENSITY_CORRECTION LOGP DEFAULT DEFAULT
MIN_CAVITY_VOLUME 10
CAVITY_RADIUS 100
PEAK_FITTING 0
PEAK_FITTING_NCYCLES 1
MIN_PEAK_HEIGHT 0
PEAK_FITTING_REFINE 0
MOLECULE_FILE "protein.pdb"
CAVITY_DETECTION 1
MIN_PSP 5
SAVE_CAVITY MESH
USE_TORSIONS 1
RMS_ACCEPT 0.1
NO_MAP_WHEN_INCOMPLETE 0
MAP_FORMAT SYBYL_ASCII_CONTOUR
FIELD_TYPE PROPENSITY
MAP_DISTRIBUTE_POINTS GAUSSIAN_SMEARING
MAX_PENETRATION 0.3
BOX_BORDER {3}
SUPERSTAR_MAP_BACKGROUND {2}
MULTIPLY_POLAR_ONLY 0
INVERSE_MAP 0
AVOID_CLOSE_GRIDPOINTS 0
MAX_NOISE_LEVEL 1
SIGNIFICANCE_LEVEL 0.001
MIN_PROPENSITY {4}
SAVE_SUPERSTAR_MAP 1
SAVE_CENTRAL_GROUPS 0
SAVE_EXCLUDED_VOLUME_MAP 0
SAVE_ACCESSIBLE_VOLUME_MAP 0
FLEXIBILITY_EXHAUSTIVE 0
FLEXIBILITY_FLAG FLAG_OPTIMISE
SMEARING_SIGMA {5}
SMEARING_NSIGMAS 2
POLAR_POLAR_CORRECTION 1
POLAR_APOLAR_CORRECTION 1
APOLAR_APOLAR_CORRECTION 9
REMOVE_BACKBONE_CONTACTS 1
REMOVE_SUSPICIOUS_CONTACTS 0
MAX_HBOND_DISTANCE 3.1
MIN_HBOND_ANGLE 100
PROPER_HBOND_DISTANCE 2.9
PROPER_HBOND_ANGLE 140
SAVE_LIGSITE_MAP 1
MIN_PEAK_DISTANCE 1
SAVE_PEAKS 1
SAVE_PEAK_MAP 0
CALC_METAL_BONDS 1
SAVE_GOLD_FITTING_POINTS 0
GOLD_MIN_PROPENSITY 5
MAX_SCATTER_ATOMS 5000
MIN_SCATTER_DENS 1.01
SURFACE_FORMAT SYBYL_MOL2
CALC_CONNOLLY_SURFACE 0
CONTOUR_LIST 2 CYAN 4 PURPLE 8 ORANGE
COVALENT_TOLERANCE 0.4
SCORE_PACKING_SHELL 0
ATOM_SCORING SINGLE_POINT'''.format(jobname,
                                    probename,
                                    settings.mapbackgroundvalue,
                                    settings.boxborder,
                                    settings.minpropensity,
                                    settings.superstar_sigma)

    return ss_str


def pymol_imports():
    out_str = """
from os.path import join
import tempfile
import zipfile
from pymol import cmd, finish_launching
from pymol.cgo import *

finish_launching()

dirpath = None
    """

    return out_str


def pymol_arrow():
    out_str = """
def cgo_arrow(atom1='pk1', atom2='pk2', radius=0.07, gap=0.0, hlength=-1, hradius=-1, color='blue red', name=''):
    from chempy import cpv
    radius, gap = float(radius), float(gap)
    hlength, hradius = float(hlength), float(hradius)

    try:
        color1, color2 = color.split()
    except:
        color1 = color2 = color
    color1 = list(cmd.get_color_tuple(color1))
    color2 = list(cmd.get_color_tuple(color2))

    def get_coord(v):
        if not isinstance(v, str):
            return v
        if v.startswith('['):
            return cmd.safe_list_eval(v)
        return cmd.get_atom_coords(v)

    xyz1 = get_coord(atom1)
    xyz2 = get_coord(atom2)
    normal = cpv.normalize(cpv.sub(xyz1, xyz2))

    if hlength < 0:
        hlength = radius * 3.0
    if hradius < 0:
        hradius = hlength * 0.6

    if gap:
        diff = cpv.scale(normal, gap)
        xyz1 = cpv.sub(xyz1, diff)
        xyz2 = cpv.add(xyz2, diff)

    xyz3 = cpv.add(cpv.scale(normal, hlength), xyz2)

    obj = [cgo.CYLINDER] + xyz1 + xyz3 + [radius] + color1 + color2 + \
          [cgo.CONE] + xyz3 + xyz2 + [hradius, 0.0] + color2 + color2 + \
          [1.0, 0.0]
    return obj

    """
    return out_str


def pymol_load_zip(zip_dir):
    out_str = """
dirpath = tempfile.mkdtemp()
zip_dir = '{0}.zip'
with zipfile.ZipFile(zip_dir) as hs_zip:
    hs_zip.extractall(dirpath)
""".format(zip_dir)

    return out_str

def pymol_protein(settings, zip_results):
    if zip_results:
        out_str = """
cmd.load(join(dirpath,"protein.pdb"), "protein")
cmd.show("cartoon", "protein")
"""
    else:
        out_str = """
cmd.load("protein.pdb", "protein")
cmd.show("cartoon", "protein")
"""
    if settings.surface:
        out_str += """
cmd.set("surface_cavity_mode", 1)
cmd.show("surface", "protein")
cmd.set("surface_trim_factor", {})
cmd.set('transparency', 0.5, "protein")
    """.format(settings.surface_trim_factor)

    return out_str


def pymol_labels(fname, objname):
    out_str = """
if dirpath:
    f = join(dirpath, "{0}")
else:
    f = "{0}"

cmd.load(f, '{1}')
cmd.hide('everything', '{1}')
cmd.label("{1}", "name")
cmd.set("label_font_id", 7)
cmd.set("label_size", -0.4)

""".format(fname, objname)
    return out_str


def pymol_grids(i, settings):
    grids = settings.grids
    threshold = settings.isosurface_threshold

    if i is not None:
        gfiles = ["{}/{}{}".format(i,p,settings.grid_extension) for p in grids]
    else:
        gfiles = ["{}{}".format(p,settings.grid_extension) for p in grids]
        i = 0

    out_str = """
colour_dict = {{'acceptor':'red', 'donor':'blue', 'apolar':'yellow', 'negative':'purple', 'positive':'cyan'}}

threshold_list = {0}
gfiles = {1}
grids = {2}
num = {3}
surf_transparency = {4}

if dirpath:
    gfiles = [join(dirpath, g) for g in gfiles]

for t in threshold_list:
    for i in range(len(grids)):
        try:
            cmd.load(r'%s'%(gfiles[i]), '%s_%s'%(grids[i], str(num)))
            cmd.isosurface('surface_%s_%s_%s'%(grids[i], t, num), '%s_%s'%(grids[i], num), t)
            cmd.set('transparency', surf_transparency, 'surface_%s_%s_%s'%(grids[i], t, num))
            cmd.color(colour_dict['%s'%(grids[i])], 'surface_%s_%s_%s'%(grids[i], t, num))
            cmd.group('threshold_%s'%(t), members = 'surface_%s_%s_%s'%(grids[i],t, num))
            cmd.group('threshold_%s' % (t), members='label_threshold_%s' % (t))
        except:
            continue



    try:
        cmd.group('hotspot_%s' % (num), members='threshold_%s' % (t))
    except:
        continue
    
    for g in grids:
        
        cmd.group('hotspot_%s' % (num), members='%s_%s' % (g,num))

""".format(threshold,
           gfiles,
           grids,
           i,
           settings.transparency)

    return out_str


def pymol_mesh(i):
    fname = join(str(i), "mesh.grd")
    n = "mesh_{}".format(i)
    out_str = """

if dirpath:
    f = join(dirpath, '{0}')
else:
    f = '{0}'
cmd.load(f, '{1}')
cmd.isomesh("iso{1}", "{1}", 0.9)
cmd.color("grey80", "iso{1}")
cmd.set('transparency', 0.4, "iso{1}")

cmd.group('hotspot_{2}', "iso{1}")
cmd.group('hotspot_{2}', "{1}")
""".format(fname, n, i)

    return out_str

def pymol_display_settings(settings):
    out_str = """
cmd.bg_color("{0}")
cmd.show("cartoon", "protein")
cmd.color("slate", "protein")
cmd.show("sticks", "organic")
cmd.hide("lines", "protein")
""".format(settings.bg_color)
    return out_str


def colourmap(scheme="inferno"):
    # more options available at https://bids.github.io/colormap/

    if scheme == "inferno":
        cm = [  #
            # [1,1,1],
            # [0.9, 0.9, 0.9],
            # [0.85, 0.85, 0.85],
            # [0.8, 0.8, 0.8],
            # [0.75, 0.75, 0.75],
            # [0.7, 0.7, 0.7],
            # [0.65, 0.65, 0.65],
            # [0.6, 0.6, 0.6],
            # [0.55, 0.55, 0.55],
            # [0.5, 0.5, 0.5],
            # [0.45, 0.45, 0.45],
            # [0.4, 0.4, 0.4],
            # [0.35, 0.35, 0.35],
            # [0.3, 0.3, 0.3],
            # [0.25, 0.25, 0.25],
            # [0.2, 0.2, 0.2],
            # [0.15, 0.15, 0.15],
            # [0.1, 0.1, 0.1],
            # [0.05, 0.05, 0.05],
            # [6.00552565e-03, 4.69194561e-03, 3.85578980e-02],
            # [7.67578856e-03, 6.13611626e-03, 4.68360336e-02],
            # [9.56051094e-03, 7.71344131e-03, 5.51430756e-02],
            # [1.16634769e-02, 9.41675403e-03, 6.34598080e-02],
            # [1.39950388e-02, 1.12247138e-02, 7.18616890e-02],
            # [1.65605595e-02, 1.31362262e-02, 8.02817951e-02],
            # [1.93732295e-02, 1.51325789e-02, 8.87668094e-02],
            # [2.24468865e-02, 1.71991484e-02, 9.73274383e-02],
            # [2.57927373e-02, 1.93306298e-02, 1.05929835e-01],
            # [2.94324251e-02, 2.15030771e-02, 1.14621328e-01],
            # [3.33852235e-02, 2.37024271e-02, 1.23397286e-01],
            # [3.76684211e-02, 2.59207864e-02, 1.32232108e-01],
            [4.22525554e-02, 2.81385015e-02, 1.41140519e-01],
            [4.69146287e-02, 3.03236129e-02, 1.50163867e-01],
            [5.16437624e-02, 3.24736172e-02, 1.59254277e-01],
            [5.64491009e-02, 3.45691867e-02, 1.68413539e-01],
            [6.13397200e-02, 3.65900213e-02, 1.77642172e-01],
            [6.63312620e-02, 3.85036268e-02, 1.86961588e-01],
            [7.14289181e-02, 4.02939095e-02, 1.96353558e-01],
            [7.66367560e-02, 4.19053329e-02, 2.05798788e-01],
            [8.19620773e-02, 4.33278666e-02, 2.15289113e-01],
            [8.74113897e-02, 4.45561662e-02, 2.24813479e-01],
            [9.29901526e-02, 4.55829503e-02, 2.34357604e-01],
            [9.87024972e-02, 4.64018731e-02, 2.43903700e-01],
            [1.04550936e-01, 4.70080541e-02, 2.53430300e-01],
            [1.10536084e-01, 4.73986708e-02, 2.62912235e-01],
            [1.16656423e-01, 4.75735920e-02, 2.72320803e-01],
            [1.22908126e-01, 4.75360183e-02, 2.81624170e-01],
            [1.29284984e-01, 4.72930838e-02, 2.90788012e-01],
            [1.35778450e-01, 4.68563678e-02, 2.99776404e-01],
            [1.42377819e-01, 4.62422566e-02, 3.08552910e-01],
            [1.49072957e-01, 4.54676444e-02, 3.17085139e-01],
            [1.55849711e-01, 4.45588056e-02, 3.25338414e-01],
            [1.62688939e-01, 4.35542881e-02, 3.33276678e-01],
            [1.69575148e-01, 4.24893149e-02, 3.40874188e-01],
            [1.76493202e-01, 4.14017089e-02, 3.48110606e-01],
            [1.83428775e-01, 4.03288858e-02, 3.54971391e-01],
            [1.90367453e-01, 3.93088888e-02, 3.61446945e-01],
            [1.97297425e-01, 3.84001825e-02, 3.67534629e-01],
            [2.04209298e-01, 3.76322609e-02, 3.73237557e-01],
            [2.11095463e-01, 3.70296488e-02, 3.78563264e-01],
            [2.17948648e-01, 3.66146049e-02, 3.83522415e-01],
            [2.24762908e-01, 3.64049901e-02, 3.88128944e-01],
            [2.31538148e-01, 3.64052511e-02, 3.92400150e-01],
            [2.38272961e-01, 3.66209949e-02, 3.96353388e-01],
            [2.44966911e-01, 3.70545017e-02, 4.00006615e-01],
            [2.51620354e-01, 3.77052832e-02, 4.03377897e-01],
            [2.58234265e-01, 3.85706153e-02, 4.06485031e-01],
            [2.64809649e-01, 3.96468666e-02, 4.09345373e-01],
            [2.71346664e-01, 4.09215821e-02, 4.11976086e-01],
            [2.77849829e-01, 4.23528741e-02, 4.14392106e-01],
            [2.84321318e-01, 4.39325787e-02, 4.16607861e-01],
            [2.90763373e-01, 4.56437598e-02, 4.18636756e-01],
            [2.97178251e-01, 4.74700293e-02, 4.20491164e-01],
            [3.03568182e-01, 4.93958927e-02, 4.22182449e-01],
            [3.09935342e-01, 5.14069729e-02, 4.23720999e-01],
            [3.16281835e-01, 5.34901321e-02, 4.25116277e-01],
            [3.22609671e-01, 5.56335178e-02, 4.26376869e-01],
            [3.28920763e-01, 5.78265505e-02, 4.27510546e-01],
            [3.35216916e-01, 6.00598734e-02, 4.28524320e-01],
            [3.41499828e-01, 6.23252772e-02, 4.29424503e-01],
            [3.47771086e-01, 6.46156100e-02, 4.30216765e-01],
            [3.54032169e-01, 6.69246832e-02, 4.30906186e-01],
            [3.60284449e-01, 6.92471753e-02, 4.31497309e-01],
            [3.66529195e-01, 7.15785403e-02, 4.31994185e-01],
            [3.72767575e-01, 7.39149211e-02, 4.32400419e-01],
            [3.79000659e-01, 7.62530701e-02, 4.32719214e-01],
            [3.85228383e-01, 7.85914864e-02, 4.32954973e-01],
            [3.91452659e-01, 8.09267058e-02, 4.33108763e-01],
            [3.97674379e-01, 8.32568129e-02, 4.33182647e-01],
            [4.03894278e-01, 8.55803445e-02, 4.33178526e-01],
            [4.10113015e-01, 8.78961593e-02, 4.33098056e-01],
            [4.16331169e-01, 9.02033992e-02, 4.32942678e-01],
            [4.22549249e-01, 9.25014543e-02, 4.32713635e-01],
            [4.28767696e-01, 9.47899342e-02, 4.32411996e-01],
            [4.34986885e-01, 9.70686417e-02, 4.32038673e-01],
            [4.41207124e-01, 9.93375510e-02, 4.31594438e-01],
            [4.47428382e-01, 1.01597079e-01, 4.31080497e-01],
            [4.53650614e-01, 1.03847716e-01, 4.30497898e-01],
            [4.59874623e-01, 1.06089165e-01, 4.29845789e-01],
            [4.66100494e-01, 1.08321923e-01, 4.29124507e-01],
            [4.72328255e-01, 1.10546584e-01, 4.28334320e-01],
            [4.78557889e-01, 1.12763831e-01, 4.27475431e-01],
            [4.84789325e-01, 1.14974430e-01, 4.26547991e-01],
            [4.91022448e-01, 1.17179219e-01, 4.25552106e-01],
            [4.97257069e-01, 1.19379132e-01, 4.24487908e-01],
            [5.03492698e-01, 1.21575414e-01, 4.23356110e-01],
            [5.09729541e-01, 1.23768654e-01, 4.22155676e-01],
            [5.15967304e-01, 1.25959947e-01, 4.20886594e-01],
            [5.22205646e-01, 1.28150439e-01, 4.19548848e-01],
            [5.28444192e-01, 1.30341324e-01, 4.18142411e-01],
            [5.34682523e-01, 1.32533845e-01, 4.16667258e-01],
            [5.40920186e-01, 1.34729286e-01, 4.15123366e-01],
            [5.47156706e-01, 1.36928959e-01, 4.13510662e-01],
            [5.53391649e-01, 1.39134147e-01, 4.11828882e-01],
            [5.59624442e-01, 1.41346265e-01, 4.10078028e-01],
            [5.65854477e-01, 1.43566769e-01, 4.08258132e-01],
            [5.72081108e-01, 1.45797150e-01, 4.06369246e-01],
            [5.78303656e-01, 1.48038934e-01, 4.04411444e-01],
            [5.84521407e-01, 1.50293679e-01, 4.02384829e-01],
            [5.90733615e-01, 1.52562977e-01, 4.00289528e-01],
            [5.96939751e-01, 1.54848232e-01, 3.98124897e-01],
            [6.03138930e-01, 1.57151161e-01, 3.95891308e-01],
            [6.09330184e-01, 1.59473549e-01, 3.93589349e-01],
            [6.15512627e-01, 1.61817111e-01, 3.91219295e-01],
            [6.21685340e-01, 1.64183582e-01, 3.88781456e-01],
            [6.27847374e-01, 1.66574724e-01, 3.86276180e-01],
            [6.33997746e-01, 1.68992314e-01, 3.83703854e-01],
            [6.40135447e-01, 1.71438150e-01, 3.81064906e-01],
            [6.46259648e-01, 1.73913876e-01, 3.78358969e-01],
            [6.52369348e-01, 1.76421271e-01, 3.75586209e-01],
            [6.58463166e-01, 1.78962399e-01, 3.72748214e-01],
            [6.64539964e-01, 1.81539111e-01, 3.69845599e-01],
            [6.70598572e-01, 1.84153268e-01, 3.66879025e-01],
            [6.76637795e-01, 1.86806728e-01, 3.63849195e-01],
            [6.82656407e-01, 1.89501352e-01, 3.60756856e-01],
            [6.88653158e-01, 1.92238994e-01, 3.57602797e-01],
            [6.94626769e-01, 1.95021500e-01, 3.54387853e-01],
            [7.00575937e-01, 1.97850703e-01, 3.51112900e-01],
            [7.06499709e-01, 2.00728196e-01, 3.47776863e-01],
            [7.12396345e-01, 2.03656029e-01, 3.44382594e-01],
            [7.18264447e-01, 2.06635993e-01, 3.40931208e-01],
            [7.24102613e-01, 2.09669834e-01, 3.37423766e-01],
            [7.29909422e-01, 2.12759270e-01, 3.33861367e-01],
            [7.35683432e-01, 2.15905976e-01, 3.30245147e-01],
            [7.41423185e-01, 2.19111589e-01, 3.26576275e-01],
            [7.47127207e-01, 2.22377697e-01, 3.22855952e-01],
            [7.52794009e-01, 2.25705837e-01, 3.19085410e-01],
            [7.58422090e-01, 2.29097492e-01, 3.15265910e-01],
            [7.64009940e-01, 2.32554083e-01, 3.11398734e-01],
            [7.69556038e-01, 2.36076967e-01, 3.07485188e-01],
            [7.75058888e-01, 2.39667435e-01, 3.03526312e-01],
            [7.80517023e-01, 2.43326720e-01, 2.99522665e-01],
            [7.85928794e-01, 2.47055968e-01, 2.95476756e-01],
            [7.91292674e-01, 2.50856232e-01, 2.91389943e-01],
            [7.96607144e-01, 2.54728485e-01, 2.87263585e-01],
            [8.01870689e-01, 2.58673610e-01, 2.83099033e-01],
            [8.07081807e-01, 2.62692401e-01, 2.78897629e-01],
            [8.12239008e-01, 2.66785558e-01, 2.74660698e-01],
            [8.17340818e-01, 2.70953688e-01, 2.70389545e-01],
            [8.22385784e-01, 2.75197300e-01, 2.66085445e-01],
            [8.27372474e-01, 2.79516805e-01, 2.61749643e-01],
            [8.32299481e-01, 2.83912516e-01, 2.57383341e-01],
            [8.37165425e-01, 2.88384647e-01, 2.52987700e-01],
            [8.41968959e-01, 2.92933312e-01, 2.48563825e-01],
            [8.46708768e-01, 2.97558528e-01, 2.44112767e-01],
            [8.51383572e-01, 3.02260213e-01, 2.39635512e-01],
            [8.55992130e-01, 3.07038188e-01, 2.35132978e-01],
            [8.60533241e-01, 3.11892183e-01, 2.30606009e-01],
            [8.65005747e-01, 3.16821833e-01, 2.26055368e-01],
            [8.69408534e-01, 3.21826685e-01, 2.21481734e-01],
            [8.73740530e-01, 3.26906201e-01, 2.16885699e-01],
            [8.78000715e-01, 3.32059760e-01, 2.12267762e-01],
            [8.82188112e-01, 3.37286663e-01, 2.07628326e-01],
            [8.86301795e-01, 3.42586137e-01, 2.02967696e-01],
            [8.90340885e-01, 3.47957340e-01, 1.98286080e-01],
            [8.94304553e-01, 3.53399363e-01, 1.93583583e-01],
            [8.98192017e-01, 3.58911240e-01, 1.88860212e-01],
            [9.02002544e-01, 3.64491949e-01, 1.84115876e-01],
            [9.05735448e-01, 3.70140419e-01, 1.79350388e-01],
            [9.09390090e-01, 3.75855533e-01, 1.74563472e-01],
            [9.12965874e-01, 3.81636138e-01, 1.69754764e-01],
            [9.16462251e-01, 3.87481044e-01, 1.64923826e-01],
            [9.19878710e-01, 3.93389034e-01, 1.60070152e-01],
            [9.23214783e-01, 3.99358867e-01, 1.55193185e-01],
            [9.26470039e-01, 4.05389282e-01, 1.50292329e-01],
            [9.29644083e-01, 4.11479007e-01, 1.45366973e-01],
            [9.32736555e-01, 4.17626756e-01, 1.40416519e-01],
            [9.35747126e-01, 4.23831237e-01, 1.35440416e-01],
            [9.38675494e-01, 4.30091162e-01, 1.30438175e-01],
            [9.41521384e-01, 4.36405243e-01, 1.25409440e-01],
            [9.44284543e-01, 4.42772199e-01, 1.20354038e-01],
            [9.46964741e-01, 4.49190757e-01, 1.15272059e-01],
            [9.49561766e-01, 4.55659658e-01, 1.10163947e-01],
            [9.52075421e-01, 4.62177656e-01, 1.05030614e-01],
            [9.54505523e-01, 4.68743522e-01, 9.98735931e-02],
            [9.56851903e-01, 4.75356048e-01, 9.46952268e-02],
            [9.59114397e-01, 4.82014044e-01, 8.94989073e-02],
            [9.61292850e-01, 4.88716345e-01, 8.42893891e-02],
            [9.63387110e-01, 4.95461806e-01, 7.90731907e-02],
            [9.65397031e-01, 5.02249309e-01, 7.38591143e-02],
            [9.67322465e-01, 5.09077761e-01, 6.86589199e-02],
            [9.69163264e-01, 5.15946092e-01, 6.34881971e-02],
            [9.70919277e-01, 5.22853259e-01, 5.83674890e-02],
            [9.72590351e-01, 5.29798246e-01, 5.33237243e-02],
            [9.74176327e-01, 5.36780059e-01, 4.83920090e-02],
            [9.75677038e-01, 5.43797733e-01, 4.36177922e-02],
            [9.77092313e-01, 5.50850323e-01, 3.90500131e-02],
            [9.78421971e-01, 5.57936911e-01, 3.49306227e-02],
            [9.79665824e-01, 5.65056600e-01, 3.14091591e-02],
            [9.80823673e-01, 5.72208516e-01, 2.85075931e-02],
            [9.81895311e-01, 5.79391803e-01, 2.62497353e-02],
            [9.82880522e-01, 5.86605627e-01, 2.46613416e-02],
            [9.83779081e-01, 5.93849168e-01, 2.37702263e-02],
            [9.84590755e-01, 6.01121626e-01, 2.36063833e-02],
            [9.85315301e-01, 6.08422211e-01, 2.42021174e-02],
            [9.85952471e-01, 6.15750147e-01, 2.55921853e-02],
            [9.86502013e-01, 6.23104667e-01, 2.78139496e-02],
            [9.86963670e-01, 6.30485011e-01, 3.09075459e-02],
            [9.87337182e-01, 6.37890424e-01, 3.49160639e-02],
            [9.87622296e-01, 6.45320152e-01, 3.98857472e-02],
            [9.87818759e-01, 6.52773439e-01, 4.55808037e-02],
            [9.87926330e-01, 6.60249526e-01, 5.17503867e-02],
            [9.87944783e-01, 6.67747641e-01, 5.83286889e-02],
            [9.87873910e-01, 6.75267000e-01, 6.52570167e-02],
            [9.87713535e-01, 6.82806802e-01, 7.24892330e-02],
            [9.87463516e-01, 6.90366218e-01, 7.99897176e-02],
            [9.87123759e-01, 6.97944391e-01, 8.77314215e-02],
            [9.86694229e-01, 7.05540424e-01, 9.56941797e-02],
            [9.86174970e-01, 7.13153375e-01, 1.03863324e-01],
            [9.85565739e-01, 7.20782460e-01, 1.12228756e-01],
            [9.84865203e-01, 7.28427497e-01, 1.20784651e-01],
            [9.84075129e-01, 7.36086521e-01, 1.29526579e-01],
            [9.83195992e-01, 7.43758326e-01, 1.38453063e-01],
            [9.82228463e-01, 7.51441596e-01, 1.47564573e-01],
            [9.81173457e-01, 7.59134892e-01, 1.56863224e-01],
            [9.80032178e-01, 7.66836624e-01, 1.66352544e-01],
            [9.78806183e-01, 7.74545028e-01, 1.76037298e-01],
            [9.77497453e-01, 7.82258138e-01, 1.85923357e-01],
            [9.76108474e-01, 7.89973753e-01, 1.96017589e-01],
            [9.74637842e-01, 7.97691563e-01, 2.06331925e-01],
            [9.73087939e-01, 8.05409333e-01, 2.16876839e-01],
            [9.71467822e-01, 8.13121725e-01, 2.27658046e-01],
            [9.69783146e-01, 8.20825143e-01, 2.38685942e-01],
            [9.68040817e-01, 8.28515491e-01, 2.49971582e-01],
            [9.66242589e-01, 8.36190976e-01, 2.61533898e-01],
            [9.64393924e-01, 8.43848069e-01, 2.73391112e-01],
            [9.62516656e-01, 8.51476340e-01, 2.85545675e-01],
            [9.60625545e-01, 8.59068716e-01, 2.98010219e-01],
            [9.58720088e-01, 8.66624355e-01, 3.10820466e-01],
            [9.56834075e-01, 8.74128569e-01, 3.23973947e-01],
            [9.54997177e-01, 8.81568926e-01, 3.37475479e-01],
            [9.53215092e-01, 8.88942277e-01, 3.51368713e-01],
            [9.51546225e-01, 8.96225909e-01, 3.65627005e-01],
            [9.50018481e-01, 9.03409063e-01, 3.80271225e-01],
            [9.48683391e-01, 9.10472964e-01, 3.95289169e-01],
            [9.47594362e-01, 9.17399053e-01, 4.10665194e-01],
            [9.46809163e-01, 9.24168246e-01, 4.26373236e-01],
            [9.46391536e-01, 9.30760752e-01, 4.42367495e-01],
            [9.46402951e-01, 9.37158971e-01, 4.58591507e-01],
            [9.46902568e-01, 9.43347775e-01, 4.74969778e-01],
            [9.47936825e-01, 9.49317522e-01, 4.91426053e-01],
            [9.49544830e-01, 9.55062900e-01, 5.07859649e-01],
            [9.51740304e-01, 9.60586693e-01, 5.24203026e-01],
            [9.54529281e-01, 9.65895868e-01, 5.40360752e-01],
            [9.57896053e-01, 9.71003330e-01, 5.56275090e-01],
            [9.61812020e-01, 9.75924241e-01, 5.71925382e-01],
            [9.66248822e-01, 9.80678193e-01, 5.87205773e-01],
            [9.71161622e-01, 9.85282161e-01, 6.02154330e-01],
            [9.76510983e-01, 9.89753437e-01, 6.16760413e-01],
            [9.82257307e-01, 9.94108844e-01, 6.31017009e-01],
            [9.88362068e-01, 9.98364143e-01, 6.44924005e-01]]

    if scheme == "magenta":
        cm = [[1, 1, 1], [1.46159096e-03, 4.66127766e-04, 1.38655200e-02],
              [2.25764007e-03, 1.29495431e-03, 1.83311461e-02],
              [3.27943222e-03, 2.30452991e-03, 2.37083291e-02],
              [4.51230222e-03, 3.49037666e-03, 2.99647059e-02],
              [5.94976987e-03, 4.84285000e-03, 3.71296695e-02],
              [7.58798550e-03, 6.35613622e-03, 4.49730774e-02],
              [9.42604390e-03, 8.02185006e-03, 5.28443561e-02],
              [1.14654337e-02, 9.82831486e-03, 6.07496380e-02],
              [1.37075706e-02, 1.17705913e-02, 6.86665843e-02],
              [1.61557566e-02, 1.38404966e-02, 7.66026660e-02],
              [1.88153670e-02, 1.60262753e-02, 8.45844897e-02],
              [2.16919340e-02, 1.83201254e-02, 9.26101050e-02],
              [2.47917814e-02, 2.07147875e-02, 1.00675555e-01],
              [2.81228154e-02, 2.32009284e-02, 1.08786954e-01],
              [3.16955304e-02, 2.57651161e-02, 1.16964722e-01],
              [3.55204468e-02, 2.83974570e-02, 1.25209396e-01],
              [3.96084872e-02, 3.10895652e-02, 1.33515085e-01],
              [4.38295350e-02, 3.38299885e-02, 1.41886249e-01],
              [4.80616391e-02, 3.66066101e-02, 1.50326989e-01],
              [5.23204388e-02, 3.94066020e-02, 1.58841025e-01],
              [5.66148978e-02, 4.21598925e-02, 1.67445592e-01],
              [6.09493930e-02, 4.47944924e-02, 1.76128834e-01],
              [6.53301801e-02, 4.73177796e-02, 1.84891506e-01],
              [6.97637296e-02, 4.97264666e-02, 1.93735088e-01],
              [7.42565152e-02, 5.20167766e-02, 2.02660374e-01],
              [7.88150034e-02, 5.41844801e-02, 2.11667355e-01],
              [8.34456313e-02, 5.62249365e-02, 2.20755099e-01],
              [8.81547730e-02, 5.81331465e-02, 2.29921611e-01],
              [9.29486914e-02, 5.99038167e-02, 2.39163669e-01],
              [9.78334770e-02, 6.15314414e-02, 2.48476662e-01],
              [1.02814972e-01, 6.30104053e-02, 2.57854400e-01],
              [1.07898679e-01, 6.43351102e-02, 2.67288933e-01],
              [1.13094451e-01, 6.54920358e-02, 2.76783978e-01],
              [1.18405035e-01, 6.64791593e-02, 2.86320656e-01],
              [1.23832651e-01, 6.72946449e-02, 2.95879431e-01],
              [1.29380192e-01, 6.79349264e-02, 3.05442931e-01],
              [1.35053322e-01, 6.83912798e-02, 3.14999890e-01],
              [1.40857952e-01, 6.86540710e-02, 3.24537640e-01],
              [1.46785234e-01, 6.87382323e-02, 3.34011109e-01],
              [1.52839217e-01, 6.86368599e-02, 3.43404450e-01],
              [1.59017511e-01, 6.83540225e-02, 3.52688028e-01],
              [1.65308131e-01, 6.79108689e-02, 3.61816426e-01],
              [1.71713033e-01, 6.73053260e-02, 3.70770827e-01],
              [1.78211730e-01, 6.65758073e-02, 3.79497161e-01],
              [1.84800877e-01, 6.57324381e-02, 3.87972507e-01],
              [1.91459745e-01, 6.48183312e-02, 3.96151969e-01],
              [1.98176877e-01, 6.38624166e-02, 4.04008953e-01],
              [2.04934882e-01, 6.29066192e-02, 4.11514273e-01],
              [2.11718061e-01, 6.19917876e-02, 4.18646741e-01],
              [2.18511590e-01, 6.11584918e-02, 4.25391816e-01],
              [2.25302032e-01, 6.04451843e-02, 4.31741767e-01],
              [2.32076515e-01, 5.98886855e-02, 4.37694665e-01],
              [2.38825991e-01, 5.95170384e-02, 4.43255999e-01],
              [2.45543175e-01, 5.93524384e-02, 4.48435938e-01],
              [2.52220252e-01, 5.94147119e-02, 4.53247729e-01],
              [2.58857304e-01, 5.97055998e-02, 4.57709924e-01],
              [2.65446744e-01, 6.02368754e-02, 4.61840297e-01],
              [2.71994089e-01, 6.09935552e-02, 4.65660375e-01],
              [2.78493300e-01, 6.19778136e-02, 4.69190328e-01],
              [2.84951097e-01, 6.31676261e-02, 4.72450879e-01],
              [2.91365817e-01, 6.45534486e-02, 4.75462193e-01],
              [2.97740413e-01, 6.61170432e-02, 4.78243482e-01],
              [3.04080941e-01, 6.78353452e-02, 4.80811572e-01],
              [3.10382027e-01, 6.97024767e-02, 4.83186340e-01],
              [3.16654235e-01, 7.16895272e-02, 4.85380429e-01],
              [3.22899126e-01, 7.37819504e-02, 4.87408399e-01],
              [3.29114038e-01, 7.59715081e-02, 4.89286796e-01],
              [3.35307503e-01, 7.82361045e-02, 4.91024144e-01],
              [3.41481725e-01, 8.05635079e-02, 4.92631321e-01],
              [3.47635742e-01, 8.29463512e-02, 4.94120923e-01],
              [3.53773161e-01, 8.53726329e-02, 4.95501096e-01],
              [3.59897941e-01, 8.78311772e-02, 4.96778331e-01],
              [3.66011928e-01, 9.03143031e-02, 4.97959963e-01],
              [3.72116205e-01, 9.28159917e-02, 4.99053326e-01],
              [3.78210547e-01, 9.53322947e-02, 5.00066568e-01],
              [3.84299445e-01, 9.78549106e-02, 5.01001964e-01],
              [3.90384361e-01, 1.00379466e-01, 5.01864236e-01],
              [3.96466670e-01, 1.02902194e-01, 5.02657590e-01],
              [4.02547663e-01, 1.05419865e-01, 5.03385761e-01],
              [4.08628505e-01, 1.07929771e-01, 5.04052118e-01],
              [4.14708664e-01, 1.10431177e-01, 5.04661843e-01],
              [4.20791157e-01, 1.12920210e-01, 5.05214935e-01],
              [4.26876965e-01, 1.15395258e-01, 5.05713602e-01],
              [4.32967001e-01, 1.17854987e-01, 5.06159754e-01],
              [4.39062114e-01, 1.20298314e-01, 5.06555026e-01],
              [4.45163096e-01, 1.22724371e-01, 5.06900806e-01],
              [4.51270678e-01, 1.25132484e-01, 5.07198258e-01],
              [4.57385535e-01, 1.27522145e-01, 5.07448336e-01],
              [4.63508291e-01, 1.29892998e-01, 5.07651812e-01],
              [4.69639514e-01, 1.32244819e-01, 5.07809282e-01],
              [4.75779723e-01, 1.34577500e-01, 5.07921193e-01],
              [4.81928997e-01, 1.36891390e-01, 5.07988509e-01],
              [4.88088169e-01, 1.39186217e-01, 5.08010737e-01],
              [4.94257673e-01, 1.41462106e-01, 5.07987836e-01],
              [5.00437834e-01, 1.43719323e-01, 5.07919772e-01],
              [5.06628929e-01, 1.45958202e-01, 5.07806420e-01],
              [5.12831195e-01, 1.48179144e-01, 5.07647570e-01],
              [5.19044825e-01, 1.50382611e-01, 5.07442938e-01],
              [5.25269968e-01, 1.52569121e-01, 5.07192172e-01],
              [5.31506735e-01, 1.54739247e-01, 5.06894860e-01],
              [5.37755194e-01, 1.56893613e-01, 5.06550538e-01],
              [5.44015371e-01, 1.59032895e-01, 5.06158696e-01],
              [5.50287252e-01, 1.61157816e-01, 5.05718782e-01],
              [5.56570783e-01, 1.63269149e-01, 5.05230210e-01],
              [5.62865867e-01, 1.65367714e-01, 5.04692365e-01],
              [5.69172368e-01, 1.67454379e-01, 5.04104606e-01],
              [5.75490107e-01, 1.69530062e-01, 5.03466273e-01],
              [5.81818864e-01, 1.71595728e-01, 5.02776690e-01],
              [5.88158375e-01, 1.73652392e-01, 5.02035167e-01],
              [5.94508337e-01, 1.75701122e-01, 5.01241011e-01],
              [6.00868399e-01, 1.77743036e-01, 5.00393522e-01],
              [6.07238169e-01, 1.79779309e-01, 4.99491999e-01],
              [6.13617209e-01, 1.81811170e-01, 4.98535746e-01],
              [6.20005032e-01, 1.83839907e-01, 4.97524075e-01],
              [6.26401108e-01, 1.85866869e-01, 4.96456304e-01],
              [6.32804854e-01, 1.87893468e-01, 4.95331769e-01],
              [6.39215638e-01, 1.89921182e-01, 4.94149821e-01],
              [6.45632778e-01, 1.91951556e-01, 4.92909832e-01],
              [6.52055535e-01, 1.93986210e-01, 4.91611196e-01],
              [6.58483116e-01, 1.96026835e-01, 4.90253338e-01],
              [6.64914668e-01, 1.98075202e-01, 4.88835712e-01],
              [6.71349279e-01, 2.00133166e-01, 4.87357807e-01],
              [6.77785975e-01, 2.02202663e-01, 4.85819154e-01],
              [6.84223712e-01, 2.04285721e-01, 4.84219325e-01],
              [6.90661380e-01, 2.06384461e-01, 4.82557941e-01],
              [6.97097796e-01, 2.08501100e-01, 4.80834678e-01],
              [7.03531700e-01, 2.10637956e-01, 4.79049270e-01],
              [7.09961888e-01, 2.12797337e-01, 4.77201121e-01],
              [7.16387038e-01, 2.14981693e-01, 4.75289780e-01],
              [7.22805451e-01, 2.17193831e-01, 4.73315708e-01],
              [7.29215521e-01, 2.19436516e-01, 4.71278924e-01],
              [7.35615545e-01, 2.21712634e-01, 4.69179541e-01],
              [7.42003713e-01, 2.24025196e-01, 4.67017774e-01],
              [7.48378107e-01, 2.26377345e-01, 4.64793954e-01],
              [7.54736692e-01, 2.28772352e-01, 4.62508534e-01],
              [7.61077312e-01, 2.31213625e-01, 4.60162106e-01],
              [7.67397681e-01, 2.33704708e-01, 4.57755411e-01],
              [7.73695380e-01, 2.36249283e-01, 4.55289354e-01],
              [7.79967847e-01, 2.38851170e-01, 4.52765022e-01],
              [7.86212372e-01, 2.41514325e-01, 4.50183695e-01],
              [7.92426972e-01, 2.44242250e-01, 4.47543155e-01],
              [7.98607760e-01, 2.47039798e-01, 4.44848441e-01],
              [8.04751511e-01, 2.49911350e-01, 4.42101615e-01],
              [8.10854841e-01, 2.52861399e-01, 4.39304963e-01],
              [8.16914186e-01, 2.55894550e-01, 4.36461074e-01],
              [8.22925797e-01, 2.59015505e-01, 4.33572874e-01],
              [8.28885740e-01, 2.62229049e-01, 4.30643647e-01],
              [8.34790818e-01, 2.65539703e-01, 4.27671352e-01],
              [8.40635680e-01, 2.68952874e-01, 4.24665620e-01],
              [8.46415804e-01, 2.72473491e-01, 4.21631064e-01],
              [8.52126490e-01, 2.76106469e-01, 4.18572767e-01],
              [8.57762870e-01, 2.79856666e-01, 4.15496319e-01],
              [8.63320397e-01, 2.83729003e-01, 4.12402889e-01],
              [8.68793368e-01, 2.87728205e-01, 4.09303002e-01],
              [8.74176342e-01, 2.91858679e-01, 4.06205397e-01],
              [8.79463944e-01, 2.96124596e-01, 4.03118034e-01],
              [8.84650824e-01, 3.00530090e-01, 4.00047060e-01],
              [8.89731418e-01, 3.05078817e-01, 3.97001559e-01],
              [8.94700194e-01, 3.09773445e-01, 3.93994634e-01],
              [8.99551884e-01, 3.14616425e-01, 3.91036674e-01],
              [9.04281297e-01, 3.19609981e-01, 3.88136889e-01],
              [9.08883524e-01, 3.24755126e-01, 3.85308008e-01],
              [9.13354091e-01, 3.30051947e-01, 3.82563414e-01],
              [9.17688852e-01, 3.35500068e-01, 3.79915138e-01],
              [9.21884187e-01, 3.41098112e-01, 3.77375977e-01],
              [9.25937102e-01, 3.46843685e-01, 3.74959077e-01],
              [9.29845090e-01, 3.52733817e-01, 3.72676513e-01],
              [9.33606454e-01, 3.58764377e-01, 3.70540883e-01],
              [9.37220874e-01, 3.64929312e-01, 3.68566525e-01],
              [9.40687443e-01, 3.71224168e-01, 3.66761699e-01],
              [9.44006448e-01, 3.77642889e-01, 3.65136328e-01],
              [9.47179528e-01, 3.84177874e-01, 3.63701130e-01],
              [9.50210150e-01, 3.90819546e-01, 3.62467694e-01],
              [9.53099077e-01, 3.97562894e-01, 3.61438431e-01],
              [9.55849237e-01, 4.04400213e-01, 3.60619076e-01],
              [9.58464079e-01, 4.11323666e-01, 3.60014232e-01],
              [9.60949221e-01, 4.18323245e-01, 3.59629789e-01],
              [9.63310281e-01, 4.25389724e-01, 3.59469020e-01],
              [9.65549351e-01, 4.32518707e-01, 3.59529151e-01],
              [9.67671128e-01, 4.39702976e-01, 3.59810172e-01],
              [9.69680441e-01, 4.46935635e-01, 3.60311120e-01],
              [9.71582181e-01, 4.54210170e-01, 3.61030156e-01],
              [9.73381238e-01, 4.61520484e-01, 3.61964652e-01],
              [9.75082439e-01, 4.68860936e-01, 3.63111292e-01],
              [9.76690494e-01, 4.76226350e-01, 3.64466162e-01],
              [9.78209957e-01, 4.83612031e-01, 3.66024854e-01],
              [9.79645181e-01, 4.91013764e-01, 3.67782559e-01],
              [9.81000291e-01, 4.98427800e-01, 3.69734157e-01],
              [9.82279159e-01, 5.05850848e-01, 3.71874301e-01],
              [9.83485387e-01, 5.13280054e-01, 3.74197501e-01],
              [9.84622298e-01, 5.20712972e-01, 3.76698186e-01],
              [9.85692925e-01, 5.28147545e-01, 3.79370774e-01],
              [9.86700017e-01, 5.35582070e-01, 3.82209724e-01],
              [9.87646038e-01, 5.43015173e-01, 3.85209578e-01],
              [9.88533173e-01, 5.50445778e-01, 3.88365009e-01],
              [9.89363341e-01, 5.57873075e-01, 3.91670846e-01],
              [9.90138201e-01, 5.65296495e-01, 3.95122099e-01],
              [9.90871208e-01, 5.72706259e-01, 3.98713971e-01],
              [9.91558165e-01, 5.80106828e-01, 4.02441058e-01],
              [9.92195728e-01, 5.87501706e-01, 4.06298792e-01],
              [9.92784669e-01, 5.94891088e-01, 4.10282976e-01],
              [9.93325561e-01, 6.02275297e-01, 4.14389658e-01],
              [9.93834412e-01, 6.09643540e-01, 4.18613221e-01],
              [9.94308514e-01, 6.16998953e-01, 4.22949672e-01],
              [9.94737698e-01, 6.24349657e-01, 4.27396771e-01],
              [9.95121854e-01, 6.31696376e-01, 4.31951492e-01],
              [9.95480469e-01, 6.39026596e-01, 4.36607159e-01],
              [9.95809924e-01, 6.46343897e-01, 4.41360951e-01],
              [9.96095703e-01, 6.53658756e-01, 4.46213021e-01],
              [9.96341406e-01, 6.60969379e-01, 4.51160201e-01],
              [9.96579803e-01, 6.68255621e-01, 4.56191814e-01],
              [9.96774784e-01, 6.75541484e-01, 4.61314158e-01],
              [9.96925427e-01, 6.82827953e-01, 4.66525689e-01],
              [9.97077185e-01, 6.90087897e-01, 4.71811461e-01],
              [9.97186253e-01, 6.97348991e-01, 4.77181727e-01],
              [9.97253982e-01, 7.04610791e-01, 4.82634651e-01],
              [9.97325180e-01, 7.11847714e-01, 4.88154375e-01],
              [9.97350983e-01, 7.19089119e-01, 4.93754665e-01],
              [9.97350583e-01, 7.26324415e-01, 4.99427972e-01],
              [9.97341259e-01, 7.33544671e-01, 5.05166839e-01],
              [9.97284689e-01, 7.40771893e-01, 5.10983331e-01],
              [9.97228367e-01, 7.47980563e-01, 5.16859378e-01],
              [9.97138480e-01, 7.55189852e-01, 5.22805996e-01],
              [9.97019342e-01, 7.62397883e-01, 5.28820775e-01],
              [9.96898254e-01, 7.69590975e-01, 5.34892341e-01],
              [9.96726862e-01, 7.76794860e-01, 5.41038571e-01],
              [9.96570645e-01, 7.83976508e-01, 5.47232992e-01],
              [9.96369065e-01, 7.91167346e-01, 5.53498939e-01],
              [9.96162309e-01, 7.98347709e-01, 5.59819643e-01],
              [9.95932448e-01, 8.05527126e-01, 5.66201824e-01],
              [9.95680107e-01, 8.12705773e-01, 5.72644795e-01],
              [9.95423973e-01, 8.19875302e-01, 5.79140130e-01],
              [9.95131288e-01, 8.27051773e-01, 5.85701463e-01],
              [9.94851089e-01, 8.34212826e-01, 5.92307093e-01],
              [9.94523666e-01, 8.41386618e-01, 5.98982818e-01],
              [9.94221900e-01, 8.48540474e-01, 6.05695903e-01],
              [9.93865767e-01, 8.55711038e-01, 6.12481798e-01],
              [9.93545285e-01, 8.62858846e-01, 6.19299300e-01],
              [9.93169558e-01, 8.70024467e-01, 6.26189463e-01],
              [9.92830963e-01, 8.77168404e-01, 6.33109148e-01],
              [9.92439881e-01, 8.84329694e-01, 6.40099465e-01],
              [9.92089454e-01, 8.91469549e-01, 6.47116021e-01],
              [9.91687744e-01, 8.98627050e-01, 6.54201544e-01],
              [9.91331929e-01, 9.05762748e-01, 6.61308839e-01],
              [9.90929685e-01, 9.12915010e-01, 6.68481201e-01],
              [9.90569914e-01, 9.20048699e-01, 6.75674592e-01],
              [9.90174637e-01, 9.27195612e-01, 6.82925602e-01],
              [9.89814839e-01, 9.34328540e-01, 6.90198194e-01],
              [9.89433736e-01, 9.41470354e-01, 6.97518628e-01],
              [9.89077438e-01, 9.48604077e-01, 7.04862519e-01],
              [9.88717064e-01, 9.55741520e-01, 7.12242232e-01],
              [9.88367028e-01, 9.62878026e-01, 7.19648627e-01],
              [9.88032885e-01, 9.70012413e-01, 7.27076773e-01],
              [9.87690702e-01, 9.77154231e-01, 7.34536205e-01],
              [9.87386827e-01, 9.84287561e-01, 7.42001547e-01],
              [9.87052509e-01, 9.91437853e-01, 7.49504188e-01]]

    return cm
